// Copyright (C) 2000 Jean-Marc Valin


#include "VisixApp.h"
#include <tree.h>
#include <parser.h>
#include "GLSpectrum.h"

static int visix_dummy = ExternalApp::addAppFactory("Visix", new _AppFactory<VisixApp>());

/*static gboolean workspace_click (GtkWidget *widget, GdkEvent *event, VisixApp *app)
{
   if (event->type == GDK_BUTTON_PRESS && ((GdkEventButton*)event)->button == 3)
   {
      //cerr << "popup\n";
      app->workspacePopup(event);
      return true;
   }
   return false;
   }*/

//extern VisixApp *app;

static void create_project(gchar * str, VisixApp *app)
{
   if (str)
   {
      string name(str);
      if (name != "")
	 app->addProject(name);
   }
}

static void add_project(GtkMenuItem     *menuitem,
                           VisixApp *app)
{
   GtkWidget *dialog = gnome_request_dialog (FALSE, "What's the project name?", "", 20, (GnomeStringCallback)create_project, app, GTK_WINDOW(app->getApp()));
   gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
   //app->addProject();
}

static void properties(GtkMenuItem     *menuitem,
                           VisixApp *app)
{
   cerr << "properties\n";
   //project->addFile();
}


static void
on_new_file1_activate                  (GtkMenuItem     *item,
                                        gpointer         user_data)
{
   VisixApp *app = (VisixApp *) gtk_object_get_data (GTK_OBJECT(item), "app");
   app->resetWorkspace();
}


/*******************  Handling the open event ******************/
static void file_open_ok_sel(GtkWidget *w, VisixApp *app)
{
   GtkWidget *ssel = GTK_WIDGET(gtk_object_get_user_data(GTK_OBJECT(w)));
   gchar *fname = g_strdup(gtk_file_selection_get_filename (GTK_FILE_SELECTION(ssel)));
        
   if (fname) {
      string name(fname);
      app->resetWorkspace();
      app->load(name);
      g_free (fname);
   }
   
   gtk_widget_destroy (GTK_WIDGET (ssel));
   ssel = NULL;
} 

static gint
file_open_destroy(GtkWidget *w, GtkWidget *sel)
{
   gtk_widget_destroy(sel);
   return TRUE;
}

static void
on_open1_activate(GtkObject *item, gpointer user_data)
{
   VisixApp *app = (VisixApp *) gtk_object_get_data (GTK_OBJECT(item), "app");
   GtkWidget *ssel = gtk_file_selection_new("Save as...");
   
   gtk_object_set_user_data(GTK_OBJECT(GTK_FILE_SELECTION(ssel)->ok_button),ssel);
   gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(ssel)->ok_button),
                      "clicked", (GtkSignalFunc)file_open_ok_sel, 
                      app);
   
   gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(ssel)->cancel_button),
                      "clicked", (GtkSignalFunc)file_open_destroy, 
                      ssel);
   
   
   gtk_widget_show(ssel);
}


/*************** Handling the save event ******************/
static void
on_save1_activate                      (GtkObject     *item,
                                        gpointer         user_data)
{
   VisixApp *app = (VisixApp *) gtk_object_get_data (GTK_OBJECT(item), "app");
   app->save();
}


static void
on_save_as1_activate                   (GtkObject     *item,
                                        gpointer         user_data)
{
   VisixApp *app = (VisixApp *) gtk_object_get_data (GTK_OBJECT(item), "app");
   app->saveAs();
}


static void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   gtk_main_quit();
}


static void
on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


static void
on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


static void
on_paste1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


static void
on_clear1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


static void
on_properties1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


static void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

static GtkWidget *create_about2 (void);

static void on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   GtkWidget *about = create_about2();
   gtk_widget_show(about);
}

static void spectrogram_add                     (GtkObject     *item,
                                        gpointer         user_data)
{
   VisixApp *app = (VisixApp *) gtk_object_get_data (GTK_OBJECT(item), "app");
   GLPlot *plot = new GLSpectrum(app->getManager());
   app->addPlot(plot);
}


static GnomeUIInfo file1_menu_uiinfo[] =
{
   GNOMEUIINFO_MENU_NEW_ITEM (N_("_New File"), NULL, on_new_file1_activate, NULL),
   GNOMEUIINFO_MENU_OPEN_ITEM (on_open1_activate, NULL),
   GNOMEUIINFO_MENU_SAVE_ITEM (on_save1_activate, NULL),
   GNOMEUIINFO_MENU_SAVE_AS_ITEM (on_save_as1_activate, NULL),
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_EXIT_ITEM (on_exit1_activate, NULL),
   GNOMEUIINFO_END
};

static GnomeUIInfo edit1_menu_uiinfo[] =
{
   GNOMEUIINFO_MENU_CUT_ITEM (on_cut1_activate, NULL),
   GNOMEUIINFO_MENU_COPY_ITEM (on_copy1_activate, NULL),
   GNOMEUIINFO_MENU_PASTE_ITEM (on_paste1_activate, NULL),
   GNOMEUIINFO_MENU_CLEAR_ITEM (on_clear1_activate, NULL),
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_PROPERTIES_ITEM (on_properties1_activate, NULL),
   GNOMEUIINFO_END
};

static GnomeUIInfo view1_menu_uiinfo[] =
{
   //GNOMEUIINFO_ITEM_NONE(_("New Spectrogram"), "Adds a new toplevel spectrogram", spectrogram_add),

   GNOMEUIINFO_ITEM_NONE("New Spectrogram", NULL, spectrogram_add),
   GNOMEUIINFO_END
};

static GnomeUIInfo settings1_menu_uiinfo[] =
{
   GNOMEUIINFO_MENU_PREFERENCES_ITEM (on_preferences1_activate, NULL),
   GNOMEUIINFO_END
};

static GnomeUIInfo help1_menu_uiinfo[] =
{
   GNOMEUIINFO_MENU_ABOUT_ITEM (on_about1_activate, NULL),
   GNOMEUIINFO_END
};

static GnomeUIInfo menubar1_uiinfo[] =
{
   GNOMEUIINFO_MENU_FILE_TREE (file1_menu_uiinfo),
   GNOMEUIINFO_MENU_EDIT_TREE (edit1_menu_uiinfo),
   GNOMEUIINFO_MENU_VIEW_TREE (view1_menu_uiinfo),
   GNOMEUIINFO_MENU_SETTINGS_TREE (settings1_menu_uiinfo),
   GNOMEUIINFO_MENU_HELP_TREE (help1_menu_uiinfo),
   GNOMEUIINFO_END
};


GtkWidget*
create_about2 (void)
{
   const gchar *authors[] = {
      "Jean-Marc Valin",
	 NULL
	 };
   GtkWidget *about2;
   
   about2 = gnome_about_new ("Visix", VERSION,
			     _("(c) Jean-Marc Valin 2000"),
			     authors,
			     _("Audio visualization software"),
			     NULL);
   gtk_object_set_data (GTK_OBJECT (about2), "about2", about2);
   gtk_window_set_modal (GTK_WINDOW (about2), TRUE);
   
   return about2;
}



VisixApp::VisixApp()
   : docName("Untitled")
   , untitled(true)
{
   //GtkWidget *app1;
   GtkWidget *dock1;
   GtkWidget *toolbar1;
   GtkWidget *tmp_toolbar_icon;
   GtkWidget *button1;
   GtkWidget *button2;
   GtkWidget *button3;
   GtkWidget *button4;
   GtkWidget *hpaned1;
   GtkWidget *tree1;
   GtkWidget *appbar1;

   app = gnome_app_new ("Visix", NULL);
   gtk_object_set_data (GTK_OBJECT (app), "app", app);

   dock1 = GNOME_APP (app)->dock;
   gtk_widget_ref (dock1);
   gtk_object_set_data_full (GTK_OBJECT (app), "dock1", dock1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (dock1);

   gnome_app_create_menus (GNOME_APP (app), menubar1_uiinfo);

   gtk_widget_ref (menubar1_uiinfo[0].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "file1",
			     menubar1_uiinfo[0].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (file1_menu_uiinfo[0].widget);
   gtk_object_set_data (GTK_OBJECT(file1_menu_uiinfo[0].widget), "app", this);
   gtk_object_set_data_full (GTK_OBJECT (app), "new_file1",
			     file1_menu_uiinfo[0].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (file1_menu_uiinfo[1].widget);
   gtk_object_set_data (GTK_OBJECT(file1_menu_uiinfo[1].widget), "app", this);
   gtk_object_set_data_full (GTK_OBJECT (app), "open1",
			     file1_menu_uiinfo[1].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (file1_menu_uiinfo[2].widget);
   gtk_object_set_data (GTK_OBJECT(file1_menu_uiinfo[2].widget), "app", this);
   gtk_object_set_data_full (GTK_OBJECT (app), "save1",
			     file1_menu_uiinfo[2].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (file1_menu_uiinfo[3].widget);
   gtk_object_set_data (GTK_OBJECT(file1_menu_uiinfo[3].widget), "app", this);
   gtk_object_set_data_full (GTK_OBJECT (app), "save_as1",
			     file1_menu_uiinfo[3].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (file1_menu_uiinfo[4].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "separator1",
			     file1_menu_uiinfo[4].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (file1_menu_uiinfo[5].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "exit1",
			     file1_menu_uiinfo[5].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (menubar1_uiinfo[1].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "edit1",
			     menubar1_uiinfo[1].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (edit1_menu_uiinfo[0].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "cut1",
			     edit1_menu_uiinfo[0].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (edit1_menu_uiinfo[1].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "copy1",
			     edit1_menu_uiinfo[1].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (edit1_menu_uiinfo[2].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "paste1",
			     edit1_menu_uiinfo[2].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (edit1_menu_uiinfo[3].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "clear1",
			     edit1_menu_uiinfo[3].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (edit1_menu_uiinfo[4].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "separator2",
			     edit1_menu_uiinfo[4].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (edit1_menu_uiinfo[5].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "properties1",
			     edit1_menu_uiinfo[5].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (menubar1_uiinfo[2].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "view1",
			     menubar1_uiinfo[2].widget,
			     (GtkDestroyNotify) gtk_widget_unref);


   gtk_widget_ref (view1_menu_uiinfo[0].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "spectrogram",
			     view1_menu_uiinfo[0].widget,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_object_set_data(GTK_OBJECT(view1_menu_uiinfo[0].widget), "app", this);

   gtk_widget_ref (menubar1_uiinfo[3].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "settings1",
			     menubar1_uiinfo[3].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (settings1_menu_uiinfo[0].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "preferences1",
			     settings1_menu_uiinfo[0].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (menubar1_uiinfo[4].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "help1",
			     menubar1_uiinfo[4].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   gtk_widget_ref (help1_menu_uiinfo[0].widget);
   gtk_object_set_data_full (GTK_OBJECT (app), "about1",
			     help1_menu_uiinfo[0].widget,
			     (GtkDestroyNotify) gtk_widget_unref);

   toolbar1 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
   gtk_widget_ref (toolbar1);
   gtk_object_set_data_full (GTK_OBJECT (app), "toolbar1", toolbar1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (toolbar1);
   gnome_app_add_toolbar (GNOME_APP (app), GTK_TOOLBAR (toolbar1), "toolbar1",
			  GNOME_DOCK_ITEM_BEH_EXCLUSIVE,
			  GNOME_DOCK_TOP, 1, 0, 0);
   gtk_container_set_border_width (GTK_CONTAINER (toolbar1), 1);
   gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbar1), 16);
   gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_SPACE_LINE);
   gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar1), GTK_RELIEF_NONE);

   tmp_toolbar_icon = gnome_stock_pixmap_widget (app, GNOME_STOCK_PIXMAP_NEW);
   button1 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
					 GTK_TOOLBAR_CHILD_BUTTON,
					 NULL,
					 _("New"),
					 _("New File"), NULL,
					 tmp_toolbar_icon, NULL, NULL);
   gtk_widget_ref (button1);
   gtk_object_set_data (GTK_OBJECT(button1), "app", this);
   gtk_object_set_data_full (GTK_OBJECT (app), "button1", button1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (button1);
   gtk_signal_connect (GTK_OBJECT (button1), "clicked",
		       GTK_SIGNAL_FUNC (on_new_file1_activate),
		       this);

   tmp_toolbar_icon = gnome_stock_pixmap_widget (app, GNOME_STOCK_PIXMAP_OPEN);
   button2 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
					 GTK_TOOLBAR_CHILD_BUTTON,
					 NULL,
					 _("Open"),
					 _("Open File"), NULL,
					 tmp_toolbar_icon, NULL, NULL);
   gtk_widget_ref (button2);
   gtk_object_set_data_full (GTK_OBJECT (app), "button2", button2,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (button2);
   gtk_object_set_data(GTK_OBJECT(button2), "app", this);
   gtk_signal_connect (GTK_OBJECT (button2), "clicked",
		       GTK_SIGNAL_FUNC (on_open1_activate),
		       this);

   tmp_toolbar_icon = gnome_stock_pixmap_widget (app, GNOME_STOCK_PIXMAP_SAVE);
   button3 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
					 GTK_TOOLBAR_CHILD_BUTTON,
					 NULL,
					 _("Save"),
					 _("Save File"), NULL,
					 tmp_toolbar_icon, NULL, NULL);
   gtk_widget_ref (button3);
   gtk_object_set_data(GTK_OBJECT(button3), "app", this);
   gtk_object_set_data_full (GTK_OBJECT (app), "button3", button3,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (button3);
   gtk_signal_connect (GTK_OBJECT (button3), "clicked",
		       GTK_SIGNAL_FUNC (on_save1_activate),
		       this);

   tmp_toolbar_icon = gnome_stock_pixmap_widget (app, GNOME_STOCK_PIXMAP_EXIT);
   button4 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
					 GTK_TOOLBAR_CHILD_BUTTON,
					 NULL,
					 _("Quit"),
					 _("Quit Visix"), NULL,
					 tmp_toolbar_icon, NULL, NULL);
   gtk_widget_ref (button4);
   gtk_object_set_data_full (GTK_OBJECT (app), "button4", button4,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (button4);
   gtk_signal_connect (GTK_OBJECT (button4), "clicked",
		       GTK_SIGNAL_FUNC (on_exit1_activate),
		       this);

   hpaned1 = gtk_hpaned_new ();
   gtk_widget_ref (hpaned1);
   gtk_object_set_data_full (GTK_OBJECT (app), "hpaned1", hpaned1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (hpaned1);
   gnome_app_set_contents (GNOME_APP (app), hpaned1);
   gtk_paned_set_position (GTK_PANED (hpaned1), 150);



   tree1 = gtk_tree_new ();
   gtk_widget_ref (tree1);
   gtk_object_set_data_full (GTK_OBJECT (app), "tree1", tree1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (tree1);
   gtk_container_add (GTK_CONTAINER (hpaned1), tree1);


   GtkWidget *tree_root = gtk_tree_item_new_with_label("Workspace");
   gtk_widget_show(tree_root);
   gtk_tree_append(GTK_TREE(tree1),tree_root);
   workspace_root = gtk_tree_new();

   gtk_tree_item_set_subtree(GTK_TREE_ITEM(tree_root), workspace_root);

  //projects.insert (projects.end(), new VisixProject(workspace_root));
  //projects.insert (projects.end(), new VisixProject(workspace_root));


  /*gtk_signal_connect (GTK_OBJECT (tree_root), "button-press-event",
		       GTK_SIGNAL_FUNC (workspace_click),
		       this);*/

   workspace_menu =  gtk_menu_new();
   gtk_widget_show(workspace_menu);


   GtkWidget *label = gtk_menu_item_new_with_label("Add Project");
   gtk_object_ref(GTK_OBJECT(label));
   gtk_object_set_data_full (GTK_OBJECT (workspace_menu), "new",
                             label,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show(label);
   gtk_menu_append(GTK_MENU(workspace_menu),label);
   gtk_signal_connect (GTK_OBJECT (label), "activate",
                       GTK_SIGNAL_FUNC (add_project),
                       this);

   label = gtk_menu_item_new_with_label("Properties");
   gtk_object_ref(GTK_OBJECT(label));
   gtk_object_set_data_full (GTK_OBJECT (workspace_menu), "new",
                             label,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show(label);
   gtk_menu_append(GTK_MENU(workspace_menu),label);
   gtk_signal_connect (GTK_OBJECT (label), "activate",
                       GTK_SIGNAL_FUNC (properties),
                       this);

   gnome_popup_menu_attach(workspace_menu, tree_root, this);

   container = hpaned1;
   /*GLPlot *plot = new GLPlot();
  gtk_container_add (GTK_CONTAINER(hpaned1), plot->getGLArea());

  gtk_widget_show (plot->getGLArea());
  */


   appbar1 = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
   gtk_widget_ref (appbar1);
   gtk_object_set_data_full (GTK_OBJECT (app), "appbar1", appbar1,
			     (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (appbar1);
   gnome_app_set_statusbar (GNOME_APP (app), appbar1);

   gtk_widget_show (app);
  
   dock = gnome_dock_new();
   gtk_widget_show(dock);
   gtk_container_add (GTK_CONTAINER(container), dock);
   //return app1;
}




void VisixApp::addPlot(GLPlot *plot)
{
   //GtkWidget *dock = gnome_dock_new();
   
   //GtkWidget *dock_item = gnome_dock_item_new("item", GNOME_DOCK_ITEM_BEH_NORMAL);
   //gtk_widget_show(dock_item);

   GtkWidget *win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_widget_show(win);
   gtk_container_add (GTK_CONTAINER(win), plot->getGLArea());
   //gtk_container_add (GTK_CONTAINER(dock_item), win);
   
   //gnome_dock_add_item(GNOME_DOCK(dock), GNOME_DOCK_ITEM(dock_item), GNOME_DOCK_LEFT, 0, 0, 0, TRUE);
   //gtk_container_add (GTK_CONTAINER(dock_item), plot->getGLArea());

   //gnome_dock_set_client_area (GNOME_DOCK(dock), plot->getGLArea());
   
   //gtk_container_add (GTK_CONTAINER(container), dock);
   //gtk_widget_show(dock);

   //gtk_container_add (GTK_CONTAINER(container), plot->getGLArea());
   
   gtk_widget_show (plot->getGLArea());
}

void VisixApp::workspacePopup(GdkEvent *event)
{
   gnome_popup_menu_do_popup_modal (workspace_menu,NULL,NULL,&(event->button),NULL);
}

void VisixApp::addProject(const string &name)
{
   projects.insert (projects.end(), new VisixProject(workspace_root, this, name));
}

void VisixApp::save()
{
   if (untitled)
   {
      saveAs();
      return;
   }
   xmlDocPtr doc;
   doc = xmlNewDoc((CHAR *)"1.0");
   doc->root = xmlNewDocNode(doc, NULL, (CHAR *)"Workspace", NULL);
   for (int i=0;i<projects.size();i++)
   {
      projects[i]->save(doc->root);
   }

   //string fullname = path+docName;
   //xmlSaveFile(fullname.c_str(), doc);
   xmlSaveFile(docName.c_str(), doc);

   xmlFreeDoc(doc);
   
}

void VisixApp::load(const string &name)
{

   xmlDocPtr doc = xmlParseFile(name.c_str());

   if (!doc || !doc->root || !doc->root->name)
   {
      cerr << "error loading\n";
      xmlFreeDoc (doc);
      return;
   }
   docName = name;
   untitled = false;

   xmlNodePtr root=doc->root;

   xmlNodePtr proj = root->childs;

   while (proj != NULL)
   {
      //cerr << "start net\n";
      if (string((char*)proj->name) == "Project")
      {
	 projects.insert(projects.end(), new VisixProject (workspace_root, this, proj));
	 //cerr << "found a project\n";
      } else {
	 cerr << "we screwed up!\n";
      }
      proj = proj->next;
   }

   xmlFreeDoc(doc);


   
}

static void file_saveas_ok_sel(GtkWidget *w, VisixApp *app)
{
   GtkWidget *ssel = GTK_WIDGET(gtk_object_get_user_data(GTK_OBJECT(w)));
   gchar *fname = g_strdup(gtk_file_selection_get_filename (GTK_FILE_SELECTION(ssel)));
        
   if (fname) {
      string name(fname);
      app->setName(name);
      app->save();
      g_free (fname);
   }
   
   gtk_widget_destroy (GTK_WIDGET (ssel));
   ssel = NULL;
} 

static gint
file_saveas_destroy(GtkWidget *w, GtkWidget *sel)
{
   gtk_widget_destroy(sel);
   return TRUE;
}




void VisixApp::saveAs()
{
   GtkWidget *ssel = gtk_file_selection_new("Save as...");
   
   gtk_object_set_user_data(GTK_OBJECT(GTK_FILE_SELECTION(ssel)->ok_button),ssel);
   gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(ssel)->ok_button),
                      "clicked", (GtkSignalFunc)file_saveas_ok_sel, 
                      this);
   
   gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(ssel)->cancel_button),
                      "clicked", (GtkSignalFunc)file_saveas_destroy, 
                      ssel);
   
   
   gtk_widget_show(ssel);
}

void VisixApp::setName(const string &_name)
{
   docName = _name;
   untitled = false;
}


void VisixApp::resetWorkspace()
{
   docName = "Untitled";
   untitled = true;
   for (int i=0;i<projects.size();i++)
   {
      delete projects[i];
   }

   projects.resize(0);
}
