// Copyright (C) 1999 Jean-Marc Valin

#include "VisixSource.h"
#include "pixmaps/wave.xpm"
#include "pixmaps/wave_select.xpm"
#include <iostream>
#include "VisixApp.h"
//extern VisixApp *app;

static gboolean source_click (GtkWidget *widget, GdkEvent *event, VisixSource *source)
{
   if (event->type == GDK_2BUTTON_PRESS && ((GdkEventButton*)event)->button == 1)
   {
      //cerr << "click!\n";
      source->toggle();
      return true;
   }
   return false;
}


static void source_select(GtkMenuItem     *menuitem,
                           VisixSource *source)
{
   source->toggle();
}

static void remove_from_project(GtkMenuItem     *menuitem,
                           VisixSource *source)
{
   source->remove();
}


static void show_properties(GtkMenuItem     *menuitem,
                           VisixSource *source)
{
   source->showProperties();
}


VisixSource::VisixSource(GtkWidget *_parent, VisixProject *_project, const string &_name)
   : parent(_parent)
   , selected(false)
   , name(_name)
   , source(NULL)
   , project(_project)
   //, properties(new AudioProperties())
   , properties(NULL)
   //, properties(new AudioProperties(*(_project->getProperties())))

{
   item = gtk_tree_item_new ();

   GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
   gtk_widget_show(hbox);
   
   //Setting up label with pixmap
   GtkWidget *label_widget = gtk_label_new ((gchar *)name.c_str());
   gtk_misc_set_alignment (GTK_MISC (label_widget), 0.0, 0.0);

   GdkBitmap *mask;
   GdkPixmap *pixmap;
   //pixmap = gdk_pixmap_create_from_xpm_d(NULL, &mask, NULL, wave_select_xpm);
   pixmap = getUnselectPix(&mask);
   pix = gtk_pixmap_new (pixmap, mask);
   
   gtk_container_add (GTK_CONTAINER (hbox), pix);
   gtk_container_add (GTK_CONTAINER (hbox), label_widget);
   gtk_container_add (GTK_CONTAINER (item), hbox);

   gtk_widget_show (label_widget);
   gtk_widget_show (pix);
   
   gtk_signal_connect (GTK_OBJECT (item), "button-press-event",
		       GTK_SIGNAL_FUNC (source_click),
		       this);
   
   gtk_widget_show(item);
   gtk_tree_append(GTK_TREE(parent),item);

   //properties->show();
   //showProperties();

   // Setting up popup menu
   menu =  gtk_menu_new();
   gtk_widget_show(menu);


   GtkWidget *label = gtk_menu_item_new_with_label(_("Select"));
   gtk_object_ref(GTK_OBJECT(label));
   gtk_object_set_data_full (GTK_OBJECT (menu), "label1",
                             label,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show(label);
   gtk_menu_append(GTK_MENU(menu),label);
   gtk_signal_connect (GTK_OBJECT (label), "activate",
                       GTK_SIGNAL_FUNC (source_select),
                       this);

   label = gtk_menu_item_new_with_label(_("Remove"));
   gtk_object_ref(GTK_OBJECT(label));
   gtk_object_set_data_full (GTK_OBJECT (menu), "label2",
                             label,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show(label);
   gtk_menu_append(GTK_MENU(menu),label);
   gtk_signal_connect (GTK_OBJECT (label), "activate",
                       GTK_SIGNAL_FUNC (remove_from_project),
                       this);

   label = gtk_menu_item_new_with_label(_("Properties"));
   gtk_object_ref(GTK_OBJECT(label));
   gtk_object_set_data_full (GTK_OBJECT (menu), "label3",
                             label,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show(label);
   gtk_menu_append(GTK_MENU(menu),label);
   gtk_signal_connect (GTK_OBJECT (label), "activate",
                       GTK_SIGNAL_FUNC (show_properties),
                       this);

   gnome_popup_menu_attach(menu, item, this);
}

VisixSource::~VisixSource()
{
   gtk_widget_destroy (menu);
   gtk_widget_destroy (item);
   delete properties;
   delete source;
}

GdkPixmap *VisixSource::getSelectPix(GdkBitmap **mask)
{
   static bool init=false;
   static GdkPixmap *pixmap;
   static GdkBitmap *theMask;
   
   if (!init)
   {
      pixmap = gdk_pixmap_create_from_xpm_d(NULL, &theMask, NULL, wave_select_xpm);
      init = true;
   }
   *mask=theMask;
   return pixmap;
}

GdkPixmap *VisixSource::getUnselectPix(GdkBitmap **mask)
{
   static bool init=false;
   static GdkPixmap *pixmap;
   static GdkBitmap *theMask;
   
   if (!init)
   {
      pixmap = gdk_pixmap_create_from_xpm_d(NULL, &theMask, NULL, wave_xpm);
      init = true;
   }
   *mask=theMask;
   return pixmap;
}



void VisixSource::toggle()
{
   selected = !selected;
   GdkPixmap *pixmap;
   GdkBitmap *mask;
   if (selected)
   {
      pixmap = getSelectPix(&mask);
      loadSource();
      //app->addSource(source);
      project->getApp()->addSource(source);
   }
   else
   {
      pixmap = getUnselectPix(&mask);
      //app->removeSource(source);
      project->getApp()->removeSource(source);
      unloadSource();
   }
   gtk_pixmap_set(GTK_PIXMAP(pix), pixmap, mask);
}

void VisixSource::showProperties()
{
   properties->show();
}


void VisixSource::remove()
{
   //app->removeSource(source);
   project->getApp()->removeSource(source);
   unloadSource();
   project->removeSource(this);
}
